package Deja;

# $Id: Deja.pm,v 1.16 2000/10/29 21:21:22 xi Exp $

###############################################################################
#    Usenet Indexer - A nice and fast usenet indexer
#    Copyright (C) 2000  Christian Laursen <xi@borderworlds.dk>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################


use strict;
use URI::Escape;
use Digest::MD5 qw(md5_hex);
require Exporter;

# Common stuff for all scripts

use vars qw(@ISA @EXPORT $natchars %limits $newsserver @newsgroups @allgroups);

@ISA = qw(Exporter);
@EXPORT = qw($natchars %limits $newsserver @newsgroups @allgroups &deja_init &lc2 &index_article &msgid &getmsgid);

# *****************
# * Configuration *
# *****************

# Common

$natchars = "";

sub lc2 ($) {
    my ($word) = @_;
    $word =~ tr/A-Z/a-z/;
    return $word;
}

%limits = (30 => 'One month',
	   60 => 'Two months',
	   183 => 'Half a year');

# News

$newsserver = "news.worldonline.dk";
@newsgroups = ("dk.admin",
	       "dk.admin.netikette",
	       "dk.admin.netmisbrug",
	       "dk.admin.opslag",
	       "dk.bolig",
	       "dk.edb",
	       "dk.edb.cad",
	       "dk.edb.database",
	       "dk.edb.database.ms-access",
	       "dk.edb.grafik",
	       "dk.edb.hardware",
	       "dk.edb.hardware.bundkort",
	       "dk.edb.hardware.drev",
	       "dk.edb.hardware.drev.cd+dvd",
	       "dk.edb.hardware.drev.cd-rom",
	       "dk.edb.hardware.drev.dvd",
	       "dk.edb.hardware.grafikkort",
	       "dk.edb.hardware.overclocking",
	       "dk.edb.hardware.server",
	       "dk.edb.internet",
	       "dk.edb.internet.software",
	       "dk.edb.internet.software.browser",
	       "dk.edb.internet.software.icq",
	       "dk.edb.internet.software.mail+news",
	       "dk.edb.internet.software.mail+news.outlook-express",
	       "dk.edb.internet.udbydere",
	       "dk.edb.internet.udbydere.ris+ros",
	       "dk.edb.internet.udbydere.webhotel",
	       "dk.edb.internet.udbydere.xdsl",
	       "dk.edb.internet.webdesign",
	       "dk.edb.internet.webdesign.clientside",
	       "dk.edb.internet.webdesign.frontpage",
	       "dk.edb.internet.webdesign.html",
	       "dk.edb.internet.webdesign.ris+ros",
	       "dk.edb.internet.webdesign.serverside",
	       "dk.edb.internet.webdesign.serverside.asp",
	       "dk.edb.mac",
	       "dk.edb.netvaerk",
	       "dk.edb.netvaerk.stort",
	       "dk.edb.netware",
	       "dk.edb.os2",
	       "dk.edb.programmering",
	       "dk.edb.programmering.basic",
	       "dk.edb.programmering.basic.visual-basic",
	       "dk.edb.programmering.c",
	       "dk.edb.programmering.java",
	       "dk.edb.programmering.pascal",
	       "dk.edb.programmering.perl",
	       "dk.edb.programpakker.ms-office",
	       "dk.edb.regneark",
	       "dk.edb.sikkerhed",
	       "dk.edb.sikkerhed.virus",
	       "dk.edb.spil",
	       "dk.edb.spil.nintendo",
	       "dk.edb.spil.playstation",
	       "dk.edb.spil.playstation.udstyr",
	       "dk.edb.spil.simulator.fly",
	       "dk.edb.sysadmin",
	       "dk.edb.system",
	       "dk.edb.system.beos",
	       "dk.edb.system.lomme-pc",
	       "dk.edb.system.ms-windows",
	       "dk.edb.system.ms-windows.desktop",
	       "dk.edb.system.ms-windows.server",
	       "dk.edb.system.unix",
	       "dk.edb.system.unix.hardware",
	       "dk.edb.tekst",
	       "dk.edb.tekst.ms-word",
	       "dk.erhverv",
	       "dk.erhverv.direkte-salg",
	       "dk.familie.adoption",
	       "dk.familie.barn",
	       "dk.forbruger",
	       "dk.forbruger.offentlig-service",
	       "dk.fritid",
	       "dk.fritid.bil",
	       "dk.fritid.boern-og-unge",
	       "dk.fritid.dykning",
	       "dk.fritid.dyr",
	       "dk.fritid.dyr.akvarium",
	       "dk.fritid.dyr.hest",
	       "dk.fritid.dyr.hund",
	       "dk.fritid.dyr.kat",
	       "dk.fritid.foto",
	       "dk.fritid.hamradio",
	       "dk.fritid.hus-og-have",
	       "dk.fritid.jagt",
	       "dk.fritid.jernbaner",
	       "dk.fritid.jernbaner.model",
	       "dk.fritid.knallert",
	       "dk.fritid.lystfiskeri",
	       "dk.fritid.motorcykel",
	       "dk.fritid.ornitologi",
	       "dk.fritid.rejse",
	       "dk.fritid.rollespil",
	       "dk.fritid.sejlads",
	       "dk.fritid.spejder",
	       "dk.general",
	       "dk.helbred",
	       "dk.helbred.behandling.alternativ",
	       "dk.helbred.handicap",
	       "dk.helbred.misbrug",
	       "dk.helbred.slank",
	       "dk.helbred.sorg+krise",
	       "dk.historie.genealogi",
	       "dk.kultur",
	       "dk.kultur.billedkunst",
	       "dk.kultur.film",
	       "dk.kultur.litteratur",
	       "dk.kultur.mad+drikke",
	       "dk.kultur.musik",
	       "dk.kultur.musik.klassisk",
	       "dk.kultur.musik.produktion",
	       "dk.kultur.sciencefiction",
	       "dk.kultur.sprog",
	       "dk.kultur.sprog.klassisk",
	       "dk.kultur.tegneserier",
	       "dk.livssyn",
	       "dk.livssyn.kristendom",
	       "dk.livssyn.newage",
	       "dk.lokalsamfund",
	       "dk.lokalsamfund.bornholm",
	       "dk.lokalsamfund.fyn",
	       "dk.lokalsamfund.oeresund",
	       "dk.marked",
	       "dk.marked.kommerciel",
	       "dk.marked.kommerciel.edb",
	       "dk.marked.kommerciel.engros",
	       "dk.marked.kommerciel.lystfiskeri",
	       "dk.marked.privat",
	       "dk.marked.privat.baad",
	       "dk.marked.privat.bil",
	       "dk.marked.privat.bilstereo",
	       "dk.marked.privat.blad+bog",
	       "dk.marked.privat.boerneudstyr",
	       "dk.marked.privat.bolig",
	       "dk.marked.privat.cykel",
	       "dk.marked.privat.dyr",
	       "dk.marked.privat.edb",
	       "dk.marked.privat.edb.mac",
	       "dk.marked.privat.edb.spil",
	       "dk.marked.privat.edb.spil.nintendo",
	       "dk.marked.privat.edb.spil.playstation",
	       "dk.marked.privat.foto",
	       "dk.marked.privat.hus-og-have",
	       "dk.marked.privat.knallert",
	       "dk.marked.privat.lystfiskeri",
	       "dk.marked.privat.moebler",
	       "dk.marked.privat.motorcykel",
	       "dk.marked.privat.musik",
	       "dk.marked.privat.radio+hifi",
	       "dk.marked.privat.telefon",
	       "dk.marked.privat.tv+video",
	       "dk.marked.privat.tv+video.dvd",
	       "dk.markedspladsen.edb",
	       "dk.medier.radio",
	       "dk.medier.satellit",
	       "dk.medier.tv",
	       "dk.natur",
	       "dk.opslag",
	       "dk.opslag.foredrag",
	       "dk.opslag.internet",
	       "dk.opslag.personlige",
	       "dk.opslag.stillinger",
	       "dk.penge",
	       "dk.penge.investering",
	       "dk.politik",
	       "dk.politik.indvandring",
	       "dk.politik.miljoe",
	       "dk.politik.trafik",
	       "dk.politik.ungdom",
	       "dk.snak",
	       "dk.snak.mudderkastning",
	       "dk.snak.seksualitet",
	       "dk.snak.snik",
	       "dk.snak.vittigheder",
	       "dk.sport",
	       "dk.sport.cykling",
	       "dk.sport.fodbold",
	       "dk.sport.ishockey",
	       "dk.sport.motorsport",
	       "dk.sport.rulleskoejter",
	       "dk.sport.squash",
	       "dk.sport.volleyball",
	       "dk.teknik.el",
	       "dk.teknik.elektronik",
	       "dk.teknik.lyd",
	       "dk.teknik.lyd.bilstereo",
	       "dk.teknik.radio.scanner",
	       "dk.teknik.satellit",
	       "dk.teknik.telefoni",
	       "dk.teknik.telefoni.isdn",
	       "dk.teknik.telefoni.mobil",
	       "dk.teknik.tv+video",
	       "dk.test",
	       "dk.undervisning",
	       "dk.undervisning.fjern",
	       "dk.undervisning.folkeskole",
	       "dk.velkommen",
	       "dk.videnskab",
	       "dk.videnskab.historie",
	       "dk.videnskab.jura",
	       "dk.videnskab.oekonomi",
	       "dk.videnskab.psykologi",
	       "dk.videnskab.religion",
	       "dk.videnskab.sundhed",
	       "dk.videnskab.teologi");

@newsgroups = ("dk.test",
	       "dk.edb.internet.webdesign.serverside");


# ******************
# * Initialization *
# ******************

@allgroups = sort @newsgroups;

mkdir ("../data", 0755) unless -d "../data";
mkdir ("../data/spool", 0755) unless -d "../data/spool";
mkdir ("../data/spool/news", 0755) unless -d "../data/spool/news";
mkdir ("../data/spool/searches", 0755) unless -d "../data/spool/searches";
mkdir ("../data/msgid", 0755) unless -d "../data/msgid";
mkdir ("../data/article", 0755) unless -d "../data/article";
mkdir ("../data/index", 0755) unless -d "../data/index";
mkdir ("../data/group", 0755) unless -d "../data/group";

# ***************
# * Common subs *
# ***************

sub deja_init {
    chdir "/home/xi/projects/deja.borderworlds.dk/docs";
}

sub index_article ($$$) {
    my ($id, $content, $groups) = @_;

    my @groups = split(/,/, $groups);
    foreach my $group (@groups) {
	open (FILE, ">>../data/group/$group") or die "Couldn't append to file: $!";
	print FILE pack("II", $id, 1);
	close FILE;
    }

    my %words = ();

    while ($content =~ m/([\w$natchars]+)/g) {
	$words{lc2($1)} += 1 if (length($1) >= 2 && length($1) <= 20);
    }

    while (my ($word, $hits) = each %words) {
	open (FILE, ">>../data/index/".uri_escape($word)) or die "Couldn't append to file: $!";
	print FILE pack("II", $id, $hits);
	close FILE;
    }
}

sub msgid {
    my ($msgid,$id) = @_;
    my $res = 0;
    
    my $file = "../data/msgid/".md5_hex($msgid);
    if (open (FILE, "<$file")) {
	while (<FILE>) {
	    if (m/^(\d+)\t(.*)$/) {
		$res = $1 if ($2 eq $msgid);
	    }
	}
	close FILE;
    }
    if (!$res && $id) {
	open (FILE, ">>$file") or die "Couldn't append to file: $!";
	print FILE "$id\t$msgid\n";
	close FILE;
    }
    return $res;
}

sub getmsgid {
    my ($id) = @_;
    my $file = sprintf ("../data/article/%08x.headers", $id);
    my $msgid;
    open (FILE, "<$file") or die "Couldn't read file: $!";
    while (<FILE>) {
	if (m/^Message-ID: (.*)$/) {
	    $msgid = $1;
	}
    }
    close FILE;
    return $msgid;
}

1;
